#include "headers/gmath.h"
#include <SDL/SDL.h>
#include <SDL/SDL_framerate.h>
#include <SDL/SDL_opengl.h>
#include <SDL/SDL_mixer.h>
#include "headers/functions.h"
#include "headers/PlayerClass.h"
#include "headers/BulletClass.h"
#include "headers/EnemyDogClass.h"
#include "headers/ParticleClass.h"
#include "headers/GoToRoomClass.h"
#include "headers/SpriteClass.h"
#include "headers/SoundClass.h"
#include "headers/defines.h" 

#define MAXTRIANGLES 600
#define MAX_LEVEL_W 640*2 
#define MAX_LEVEL_H 480*2

Sprite mysprite;
Sound mysound;
Particle myparticle[100];
Player myplayer;
GoToRoom mygotoroom;
EnemyDog myenemydog;
    
int main( int argc, char *argv[] ) 
{	
	if ( SDL_Init( SDL_INIT_EVERYTHING ) < 0 )
	    return(0);	
	
	FPSmanager manager;

	SDL_Event event;
	Uint8 *keystates = SDL_GetKeyState( NULL );	
	SDL_Surface *screen;

	screen = SDL_SetVideoMode( WINDOW_WIDTH, WINDOW_HEIGHT, 32, SDL_OPENGL );
    
	if (screen == NULL) 
    	return(0) ;
    
    OpenGL_init();    

    SDL_initFramerate( &manager );
    SDL_setFramerate( &manager, 60 ); //60 FPS
    
    SDL_Rect camera = {0, 0, WINDOW_WIDTH, WINDOW_HEIGHT};
    
	// Terrain grid
    unsigned char tile_grid[MAX_LEVEL_W/16][MAX_LEVEL_H/16], tileset;    
    unsigned int terrain_grid[MAXTRIANGLES][9], triangles, level_w, level_h, level = 1;
    
   	GLuint tex_tileset[2];    
	tex_tileset[0] = LoadTexture("resources/cave_tiles.tga");
	tex_tileset[1] = LoadTexture("resources/jungle_tiles.tga");	
	if ( !tex_tileset[0] || !tex_tileset[1] )
		return 0;
    
    load_terrain(terrain_grid, &triangles, &level_w, &level_h, tile_grid, &tileset, level);

	mysprite.load();
	mysound.load();	
	mysound.play_music(0);
    Bullet mybullet[100];
        
    int mouse_x, mouse_y;
	    
    while ( 1 )
    {				
        SDL_framerateDelay( &manager );

		// Clear the screen before drawing (in this dark blue color)
		glClearColor( 0, 0, .2, 0 );
		glClear( GL_COLOR_BUFFER_BIT );   
		
		SDL_GetMouseState( &mouse_x, &mouse_y );		
									
		//Bullet updates
		for(char i = 0; i < 100; i++)
		{
			if (!mybullet[i].get_destroyed())
			{
				mybullet[i].move(terrain_grid, triangles);
				mybullet[i].show();									
			}
		}
				
		//Enemy updates
		if (!myenemydog.get_destroyed())
		{
			myenemydog.move(terrain_grid, triangles);
			myenemydog.show(terrain_grid);
		}
		
		//GoToRoom updates
		if (!mygotoroom.get_destroyed())
		{
			int tmplevel = mygotoroom.step();
			if ( tmplevel != -1 )
			{				
				level = tmplevel;
				SDL_Rect camera = {0, 0, WINDOW_WIDTH, WINDOW_HEIGHT};				
				load_terrain(terrain_grid, &triangles, &level_w, &level_h, tile_grid, &tileset, level);
			}
		}
									
		//Player updates							
		myplayer.move(keystates, terrain_grid, triangles);
		myplayer.show(keystates, terrain_grid, mouse_x, mouse_y, camera.x, camera.y);		
		
		//Particle updates
		for(char i = 0; i < 100; i++)
		{
			if (!myparticle[i].get_destroyed())
			{
				myparticle[i].move();
				myparticle[i].show();									
			}
		}
		
        set_camera( myplayer.get_x(), &camera.x, myplayer.get_y(), &camera.y, level_w, level_h);
		
		// Draw terrain
		//draw_terrain(terrain_grid, triangles); //triangles
		draw_tiles(tile_grid, tex_tileset[tileset]);
		
	    SDL_GL_SwapBuffers();	     
					
		while ( SDL_PollEvent( &event ) ) //events handling
    	{
        	switch ( event.type ) 
        	{
        	    case SDL_QUIT:
				    SDL_FreeSurface( screen );
				    /*Mix_FreeChunk(sample[0]);
				    Mix_FreeChunk(sample[1]);
				    Mix_FreeChunk(sample[2]);*/
				    //Mix_FreeMusic(music);
				    //Mix_CloseAudio();
                	return(0);
	            break ;   
	            case SDL_KEYDOWN:
    	            switch( event.key.keysym.sym ) 
        	        {
            	        case KEY_UP:																																			
							if (myplayer.get_hanging_r() || myplayer.get_hanging_l())
							{																																						
								if (!myplayer.get_animated())
									myplayer.set_animated(1);																
							}
							else if ( myplayer.pl_place_free( terrain_grid, myplayer.get_x(), myplayer.get_y() - 16)	)				
								myplayer.jump(keystates, terrain_grid);
						break;	 
            	        case SDLK_q:
//							myplayer.switch_weapons();
						break;	        						         						  
            	        case SDLK_e:
//							myplayer.set_attacking(1);
						break;	   						
            	        case KEY_LEFT:
							if (myplayer.get_hanging_r())
							{
								do
								{
									if (myplayer.can_move(terrain_grid, myplayer.get_x() - 1, myplayer.get_y()))
									{
										myplayer.set_x(myplayer.get_x() - 1);
										myplayer.check_hanging(terrain_grid);
									}
								}
								while (myplayer.get_hanging_r());
								if (myplayer.can_move(terrain_grid, myplayer.get_x() - 6, myplayer.get_y()))
									myplayer.set_x(myplayer.get_x() - 6);																
							}
						break;	
            	        case KEY_RIGHT:
							if (myplayer.get_hanging_l())
							{
								do
								{
									if (myplayer.can_move(terrain_grid, myplayer.get_x() + 1, myplayer.get_y()))
									{
										myplayer.set_x(myplayer.get_x() + 1);
										myplayer.check_hanging(terrain_grid);
									}
								}
								while (myplayer.get_hanging_l());
								if (myplayer.can_move(terrain_grid, myplayer.get_x() + 6, myplayer.get_y()))
									myplayer.set_x(myplayer.get_x() + 6);								
							}
						break;	
						case KEY_DOWN:		
							if (myplayer.get_hanging_r())
							{								
								do
								{
									if (myplayer.can_move(terrain_grid, myplayer.get_x() - 1, myplayer.get_y()))
									{
										myplayer.set_x(myplayer.get_x() - 1);
										myplayer.check_hanging(terrain_grid);
									}
								}
								while (myplayer.get_hanging_r());
								if (myplayer.can_move(terrain_grid, myplayer.get_x() - 6, myplayer.get_y()))
									myplayer.set_x(myplayer.get_x() - 6);																
							}					
							else if (myplayer.get_hanging_l())
							{
								do
								{
									if (myplayer.can_move(terrain_grid, myplayer.get_x() + 1, myplayer.get_y()))
									{
										myplayer.set_x(myplayer.get_x() + 1);
										myplayer.check_hanging(terrain_grid);
									}
								}
								while (myplayer.get_hanging_l());
								if (myplayer.can_move(terrain_grid, myplayer.get_x() + 6, myplayer.get_y()))
									myplayer.set_x(myplayer.get_x() + 6);									
							}									
						break;								
					}
				break;	
				case SDL_MOUSEBUTTONDOWN:
            	    if ( event.button.button == SDL_BUTTON_LEFT )
					{
						if (myplayer.get_weapon() == 1 && !(myplayer.get_hanging_l() || myplayer.get_hanging_r()))
						{
							for (char i = 0; i < 100; i++)
							{
								if (mybullet[i].get_destroyed())
								{
									mybullet[i].create(myplayer.get_x(),myplayer.get_y() - 4,(point_direction((double)myplayer.get_x(),(double)myplayer.get_y(),(double)mouse_x + (double)camera.x,(double)mouse_y+ (double)camera.y)),0);
									break;
								}
							}
						}
						else
							myplayer.set_attacking(1);						
					}		
					else if ( event.button.button == SDL_BUTTON_RIGHT )
						myplayer.switch_weapons();	
    	    }
	    }	
    }
    return 0;
}


