#include "headers/gmath.h"
#include <SDL/SDL.h>
#include <SDL/SDL_framerate.h>
#include <SDL/SDL_opengl.h>
#include "headers/functions.h"

#define W 640
#define H 480
#define MAXTRIANGLES 600
#define MAX_LEVEL_W 640*2 
#define MAX_LEVEL_H 480*2

int main( int argc, char *argv[] ) 
{
	//Init SDL
	if ( SDL_Init( SDL_INIT_EVERYTHING ) < 0 )
	    return(0);	
	    
	SDL_ShowCursor(0);	    
		
	//Controlling events
	SDL_Event event;	
	
	//Controlling keystates
	Uint8 *keystates = SDL_GetKeyState( NULL );	
	
	//Screen's surface and settnigs
	SDL_Surface *screen;	
	screen = SDL_SetVideoMode( W, H, 32, SDL_OPENGL );
	if (screen == NULL) 
    	return(0);

	//Init openGL    
    OpenGL_init();    

	//Controlling FPS
	FPSmanager manager;
    SDL_initFramerate( &manager );
    SDL_setFramerate( &manager, 60 );    
    
	// Terrain grid plus some auxiliary self-explanatory variables
    unsigned char mode = 0, vertex = 1, opt = 0, objects = 100, object = 1, selected, tile_grid[MAX_LEVEL_W/16][MAX_LEVEL_H/16];
    unsigned int triangles = 1, terrain_grid[MAXTRIANGLES][9], camera_x = 0, camera_y = 0, level_w = 640, level_h = 480, objects_grid[100][3];
	int mouse_x, mouse_y, mx, my;
	
	//Clear tile_grid
	for( unsigned char i = 0; i < MAX_LEVEL_W/16; i++ ) 
		for( unsigned char j = 0; j < MAX_LEVEL_H/16; j++ ) 	
			tile_grid[i][j] = 0;
	
	//Clear objects grid		
	for ( char i = 0; i < objects; i++ )
	{
		objects_grid[i][0] = 0; //kind
		objects_grid[i][1] = 0; //x
		objects_grid[i][2] = 0; //y														
	}			
		
	//Two working modes: 0 for solid masks, 1 for tiles
	bool show_grid = 1, show_tiles = 1, show_solid = 1, show_objects = 1, mouse_left_down = 0, mouse_right_down = 0, select_tile;
    
	//Digits strip
	GLuint tex_digits, tex_tileset[2], tex_objects[2];    
	tex_digits = LoadTexture("resources/bitmap_font.tga");
	tex_tileset[0] = LoadTexture("resources/cave_tiles.tga");	
	tex_tileset[1] = LoadTexture("resources/jungle_tiles.tga");	
	tex_objects[0] = LoadTexture("resources/enemy_dog.tga");	
	tex_objects[1] = LoadTexture("resources/gotoroom.tga");	
	char tileset = 0;
	
	if (!tex_digits || !tex_tileset[0] || !tex_tileset[1] || !tex_objects[0] )
		return 0;		   

	//Tile variables
	unsigned char tile = 0;		
		
	srand(rand());
		
    //Main Loop
    while ( 1 )
    {		
		//Keep FPS				
        SDL_framerateDelay( &manager );

		//Get mouse variables x and y
		SDL_GetMouseState( &mouse_x, &mouse_y );

		//Set camera
		if ( !select_tile )
			set_camera( level_w, level_h, mouse_x, mouse_y, &camera_x, &camera_y);
    	
    	//Set projection on this matrix
		glMatrixMode( GL_PROJECTION );
	    glLoadIdentity();
    	gluOrtho2D( camera_x, camera_x + 640, camera_y + 480, camera_y ); //left, right, bottom, top
	    
	    //Draw on this other one
    	glMatrixMode( GL_MODELVIEW );
	    glLoadIdentity();

		// Clear the screen before drawing
		glClear( GL_COLOR_BUFFER_BIT );   

		//Draw current triangles
		if (show_solid || mode == 0)		
			draw_terrain(terrain_grid, triangles);

		//Show grid
		if ( show_grid )
		{	
		    glDisable(GL_TEXTURE_2D);		
			//Draw grid
			glBegin( GL_LINES );
			glColor3f(.0,.0,.5);		
			if (show_grid)
			{
				//Horizontal lines
				for( int i = 16; i < level_w; i += 16)
				{
					glVertex2f( i, 0 );
					glVertex2f( i, level_h );			
				}
				//Vertical lines
				for( int i = 16; i < level_h; i += 16)
				{
					glVertex2f( 0, i );
					glVertex2f( level_w, i );			
				}
			}
			glEnd();
		}
		glColor3f(1.0,1.0,1.0);	
	
		//Draw objects
		if ( show_objects || mode == 2 )
		{
			for ( char i = 0; i < objects; i++ )
			{
				if ( objects_grid[i][0] != 0 )
					Draw_Texture( tex_objects[objects_grid[i][0] - 1], objects_grid[i][1] - 16, objects_grid[i][2] - 16, 32, 32, 32, 32, 0 );		
			}
		}

		//Draw tiles
		if ( show_tiles || mode == 1 )
			draw_tiles(tile_grid, tex_tileset, tileset);
				
		//Bug fix
	    if (vertex > 3)
	    	vertex = 1;				  
		else if (vertex == 0)
			vertex = 1;	    	
										
		//Draw selected tile on mouse position, if on tile mode and not deleting
		if ( mode == 1 && !mouse_right_down )
			Draw_Texture( tex_tileset[tileset], mx / 16 * 16, my / 16 * 16, 16, 16, 1024, 16, tile );
		//Draw selected object on mouse position, if on object mode and not deleting
		else if ( mode == 2 && !mouse_right_down )
			Draw_Texture( tex_objects[object - 1], mx - 16, my - 16, 32, 32, 32, 32, 0 );			
		
		//Draw HUD
		select_tile = (keystates[SDLK_RCTRL] || keystates[SDLK_LCTRL]) && mode == 1;
		draw_hud(terrain_grid, triangles, vertex, camera_x, camera_y, &mouse_x, &mouse_y, level_w, level_h, tex_digits, tex_tileset[tileset], &mx, &my, mode, show_grid, show_tiles, show_solid, show_objects, opt, tile, select_tile, &selected, &tileset, object);
				
		//Time to swap buffers, once everything is drawn
	    SDL_GL_SwapBuffers();	  		
				
		//Events handling					
		while ( SDL_PollEvent( &event ) )
    	{
        	switch ( event.type ) 
        	{
        	    case SDL_QUIT:
				    SDL_FreeSurface( screen ) ;
                	return(0);
	            break ;   
	            case SDL_KEYDOWN:
    	            switch( event.key.keysym.sym ) 
        	        {
            	        case SDLK_UP:
							//Move the cursor up
							opt = (char) max( 2, (double) opt - 1.0, 0.0 );
						break;	
						case SDLK_DOWN:	
							//Move the cursor down
							opt = (char) min( 2, (double) opt + 1.0, 8.0 );
						break;							          						  
            	        case SDLK_LEFT:
							//Depending on opt's value...
							switch(opt)
							{								
								//Mode
								case 0:
									mode = (char) max( 2, (double) mode - 1.0, 0.0 );
								break;								
								//Tileset
								case 1:
									tileset = (char) max( 2, (double) tileset - 1.0, 0.0 );
								break;																
								//Object
								case 2:
									object = (char) max( 2, (double) object - 1.0, 1.0 );
								break;								
								//Grid
								case 3:
									show_grid = 0;
								break;
								//Tiles
								case 4:
									show_tiles = 0;
								break;
								//Solid
								case 5:
									show_solid = 0;
								break;
								//Solid
								case 6:
									show_objects = 0;
								break;								
								//Room width
								case 7:
									level_w = (int) max( 2, (double) level_w - 16.0, 640.0 );									
								break;
								//Room height
								case 8:
									level_h = (int) max( 2, (double) level_h - 16.0, 480.0 );									
								break;								
							}
						break;	
						case SDLK_RIGHT:
							//Depending on opt's value...
							switch(opt)
							{								
								//Mode
								case 0:
									mode = (char) min( 2, (double) mode + 1.0, 2.0 );
								break;						
								//Tileset
								case 1:
									tileset = (char) min( 2, (double) tileset + 1.0, 1.0 );
								break;
								//Object
								case 2:
									object = (char) min( 2, (double) object + 1.0, 2.0 );
								break;								
								//Grid
								case 3:
									show_grid = 1;
								break;
								//Tiles
								case 4:
									show_tiles = 1;
								break;
								//Solid
								case 5:
									show_solid = 1;
								break;
								//Show objects
								case 6:
									show_objects = 1;
								break;								
								//Room width
								case 7:
									level_w += 16;									
								break;
								//Room height
								case 8:
									level_h += 16;									
								break;								
							}
						break;
						case SDLK_RETURN:
							save_terrain(terrain_grid, triangles, level_w, level_h, tile_grid, tileset, objects_grid, objects);
						break;
						case SDLK_SPACE:
							load_terrain(terrain_grid, &triangles, &level_w, &level_h, tile_grid, &tileset, objects_grid );							
						break;	
						case SDLK_f:
							fill_tiles( tile_grid, level_w, level_h, terrain_grid, triangles);
						break;				
					}
				break;	
				case SDL_MOUSEBUTTONDOWN:
            	    if ( event.button.button == SDL_BUTTON_LEFT )
					{
						mouse_left_down = 1;
						//If on solid mask mode
						if ( !mode )
						{
							//Add mouse (more less) coordinates as new vertices
							int vx, vy;
						
							//If we clicked reasonably next to another vertex, we surely wanted to choose the vertex's exact same coordinates
							bool done = 0;
							//For every triangle
							for(int i = 1; i < triangles; i++)
							{
								//And every vertex
								for(char j = 0; j < 3; j++)
								{
									vx = terrain_grid[i][j * 2];
									vy = terrain_grid[i][j * 2 + 1];
									//If that vertex is reasonable next to ours								
									if (point_distance((double)vx, (double)vy, (double)mx, (double)my) < 16)
									{
										mx = vx;
										my = vy;
										done = 1;
										break;
									}
								}
								if (done)
									break;
							}						
					
							if (mx == level_w - 1)
								mx = level_w;
			
							//We will add vertex only if different from previously chosen vertices for current triangle
							bool add_vertex = 1;
							for(char i = 1; i < vertex; i++)
								if ((terrain_grid[triangles][(i - 1) * 2] == mx) && (terrain_grid[triangles][(i - 1) * 2 + 1] == my))
									add_vertex = 0;						
							
							//Add vertex
							if (add_vertex)
							{
								terrain_grid[triangles][(vertex - 1) * 2] = mx; //x
								terrain_grid[triangles][(vertex - 1) * 2 + 1] = my; //y
	
								if (vertex == 3)
								{
									triangles += 1;
									vertex = 1;
								}
								else
									vertex += 1;							
							}											
						}
						else if ( mode == 1 ) //tiled mode
						{
							if ( selected != 0 )
							{
								tile = selected;
								mouse_left_down = 0;
							}
						}
						else if ( mode == 2 ) //objects mode
						{
							for ( char i = 0; i < objects; i++ )
							{
								if ( objects_grid[i][0] == 0 ) //free slot
								{
									objects_grid[i][0] = object; //kind
									objects_grid[i][1] = mx; //x
									objects_grid[i][2] = my; //y		
									break;												
								}
							}
						}
					}
            	    else if ( event.button.button == SDL_BUTTON_RIGHT )
						mouse_right_down = 1;
				break;										
				case SDL_MOUSEBUTTONUP:
            	    if ( event.button.button == SDL_BUTTON_LEFT )
						mouse_left_down = 0;
            	    else if ( event.button.button == SDL_BUTTON_RIGHT )
						mouse_right_down = 0;
				break;										
    	    }
	    }
		
		if ( mouse_left_down == 1 )
		{			
			if ( mode == 1 ) //Tile adding mode
			{
				tile_grid[mx / 16][my / 16] = tile;
			}	
		}
		
		if ( mouse_right_down == 1)
		{
			//If on solid mask mode
			if (!mode)
			{
				//For every triangle
				for(int i = 1; i < triangles; i++)
				{
					//If we clicked on it
					if (in_triangle(terrain_grid, i, mx, my))
					{
						//Delete that triangle (aka copy others down)
						for(char j = i; j < triangles; j++)
						{
							//Copy three vertices down
							for(char k = 0; k < 6; k++)
								terrain_grid[j][k] = terrain_grid[j + 1][k];
						}
						triangles -= 1;	
						vertex = 1;							
						break;
					}
				}						
			}
			else if ( mode == 1 ) //tiles
			{
				tile_grid[mx / 16][my / 16] = 0;
			}
			else if ( mode == 2 ) //objects
			{
				for( char i = 0; i < objects; i++)				
				{
					if ( objects_grid[i][0] != 0 )
					{
						if ( point_distance( (double) mx, (double) my, (double) objects_grid[i][1], (double) objects_grid[i][2] ) < 16 )
							objects_grid[i][0] = 0;
					}
				}
			}
		}
		
    }
    return 0;
}
