#include "../headers/functions.h"
#include "../headers/gmath.h"
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include "../headers/tga.h"
#include <string.h>

GLuint LoadTexture(char *TexName)
{
	TGAImg Img;        // Image loader
	GLuint Texture;
 
	// Load our Texture
	if(Img.Load(TexName)!=IMG_OK)
    	return 0;
 
	glGenTextures(1, &Texture);            // Allocate space for texture
	glBindTexture(GL_TEXTURE_2D,Texture); // Set our Tex handle as current
 
	// Create the texture
	if(Img.GetBPP()==24)
    	glTexImage2D(GL_TEXTURE_2D,0,3,Img.GetWidth(),Img.GetHeight(),0,GL_RGB,GL_UNSIGNED_BYTE,Img.GetImg());
    else if(Img.GetBPP()==32)
    	glTexImage2D(GL_TEXTURE_2D,0,4,Img.GetWidth(),Img.GetHeight(),0,GL_RGBA,GL_UNSIGNED_BYTE,Img.GetImg());
    else
    	return 0;
 
   // Specify filtering and edge actions
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
 
   return Texture;
}

void OpenGL_init()
{
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1);
    
	// Set the OpenGL state after creating the context with SDL_SetVideoMode
	glClearColor( 0, 0, 0, 0 );
	
	glEnable (GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	

    glViewport( 0, 0, 640, 480);

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();

    gluOrtho2D( 0, 640, 480, 0 ); //left, right, bottom, top
    
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
}

void Draw_Texture(unsigned int texture, int x, int y, unsigned char img_w, unsigned char img_h, unsigned int tex_w, unsigned int tex_h, unsigned char img)
{
	glEnable( GL_TEXTURE_2D ); // Need this to display a texture	
	
	// Bind the texture to which subsequent calls refer to
    glBindTexture( GL_TEXTURE_2D, texture );

	glBegin( GL_QUADS );
    	
    glTexCoord2d( (double)(img * img_w) / (double) tex_w, 0 );
	glVertex3f( x, y, 0 );
    
    glTexCoord2d( ((double)((img + 1) * img_w)) / ((double)tex_w), 0 );
	glVertex3f( x + img_w, y, 0 );
    
	glTexCoord2d( ((double)((img + 1) * img_w)) / ((double)tex_w), ((double) img_h) / ((double)tex_h) );
    glVertex3f( x + img_w, y + img_h, 0 );
    
    glTexCoord2d( ((double)(img * img_w)) / ((double)tex_w), ((double) img_h) / ((double)tex_h) );
	glVertex3f( x, y + img_h, 0 );
    
	glEnd();			
}

void draw_hud(unsigned int terrain_grid[][9], unsigned char triangles, unsigned char vertex, unsigned int camera_x, unsigned int camera_y, int *mouse_x, int *mouse_y, int level_w, int level_h, GLuint tex_digits, GLuint tex_tileset, int *mx, int *my, char mode, bool show_grid, bool show_tiles, bool show_solid, bool show_objects, char opt, char tile, bool select_tile, unsigned char* selected, char * tileset, char object )
{		
	//String where we'll store gonna-be-drawn stuff
	char cadena[20]; 			
	
	//xorigin for texts (to avoid it hiding some parts of the level we are interested on)
	int xorg;
	
	//If solid mask mode
	if ( !mode )
	{
		xorg = 560 * ( *mx < 80 && *my < 20 );

		//Bg black rectangle
	    glDisable(GL_TEXTURE_2D);		
		glBegin( GL_POLYGON );
		glColor4f(.0, .0, .0, .5);			
		glVertex2f( xorg + camera_x, camera_y );
		glVertex2f( xorg + camera_x + 80, camera_y );	
		glVertex2f( xorg + camera_x + 80, camera_y + 30 );	
		glVertex2f( xorg + camera_x, camera_y + 30 );		
		glEnd();
		
		glColor3f(1.0, 1.0, 1.0);
		
		//Triangles
		sprintf(cadena,"Triangles: %d", triangles);
		draw_text(xorg + camera_x, camera_y, cadena, tex_digits);
		//Vertices
		sprintf(cadena,"Vertices: %d", vertex - 1);
		draw_text(xorg + camera_x, camera_y + 10, cadena, tex_digits);
		//Vertices
		char this_triangle = 0;
		//For every triangle
		for(char i = 1; i < triangles; i++)
		{
			//If we clicked on it
			if (in_triangle(terrain_grid, i, *mx, *my))
			{
				this_triangle = i;
				break;
			}
		}			
		sprintf(cadena,"This tri.: %d", this_triangle);
		draw_text(xorg + camera_x, camera_y + 20, cadena, tex_digits);		
		
		
		//Set points, and their attributes
		glDisable(GL_TEXTURE_2D);
		glPointSize(4.0);    	
		glBegin( GL_POINTS );
		
		//Show current vertices location
		for(char i = 1; i <= vertex; i++)
		{
			//Blue dots show them at their position
			if ( i > 1 )
			{
				glColor3f(.0,.0,1.0);			
				glVertex2f( terrain_grid[triangles][(i - 2) * 2], terrain_grid[triangles][(i - 2) * 2 + 1] );						
			}
		}	
		
		glEnd();	
	}

	//Set real x position of the mouse over the level
	*mx = *mouse_x + camera_x, *my = *mouse_y + camera_y;	

	xorg = 480 * ( *mx < 160 && *my > 340 );

	//Bg black rectangle
    glDisable(GL_TEXTURE_2D);	
	glBegin( GL_QUADS );
	glColor4f(.0, .0, .0, .5);			
	glVertex2f( xorg + camera_x, camera_y + 330 );
	glVertex2f( xorg + camera_x + 160, camera_y + 330 );	
	glVertex2f( xorg + camera_x + 160, camera_y + 480 );	
	glVertex2f( xorg + camera_x, camera_y + 480 );	
	
	if ( select_tile )
	{
		//Draw black rectangle
		glVertex2f( camera_x, camera_y + 200 );
		glVertex2f( camera_x + 640, camera_y + 200 );	
		glVertex2f( camera_x + 640, camera_y + 280 );	
		glVertex2f( camera_x, camera_y + 280 );			
	}			
	glEnd();

	*selected = 0;

	if ( select_tile )
	{	
		//Draw tiles
		glColor3f( 1.0, 1.0, 1.0 );
		for( char j = 0; j < 3; j++ )
			for( char i = 0; i < 26; i++ )	
			{		
				if  ( j * 26 + i + 1 >= 64 )
					break;
					
				//Draw green rectangle if highlighted one
				if ( *mx > camera_x + 8 + i * 24 - 1 && *mx < camera_x + 8 + i * 24 + 16 + 1 && *my > camera_y + 200 + 8 + j * 24 - 1 && *my < camera_y + 200  + 8 + j * 24 + 16 + 1 )
				{
					*selected = j * 26 + i + 1;
					glDisable(GL_TEXTURE_2D);
					glBegin( GL_LINE_LOOP );
					glColor3f(.0, 1.0, .0);
					glVertex2f( camera_x + 8 + i * 24 - 1, camera_y + 200 + 8 + j * 24 - 1 );
					glVertex2f( camera_x + 8 + i * 24 + 16 + 1, camera_y + 200  + 8 + j * 24 - 1 );	
					glVertex2f( camera_x + 8 + i * 24 + 16 + 1, camera_y + 200  + 8 + j * 24 + 16 + 1 );	
					glVertex2f( camera_x + 8 + i * 24 - 1, camera_y + 200  + 8 + j * 24 + 16 + 1 );	
					glEnd();
				}
				//Draw tile			
				glColor3f(1.0, 1.0, 1.0);	
				glEnable(GL_TEXTURE_2D);					
				Draw_Texture( tex_tileset, camera_x + 8 + i * 24, camera_y + 200 + 8 + j * 24, 16, 16, 1024, 16, j * 26 + i + 1 );		
				
			}
	}	
	
	//Draw mouse pointer
    glDisable(GL_TEXTURE_2D);
	glPointSize(3.0);    	
	glBegin( GL_POINTS );		
	glColor3f(.0,1.0,.0);			
	glVertex2f( *mx, *my );								
	glEnd();

	//Set drawing color
	glColor3f(1.0 ,1.0, 1.0);	

	//Mode
	sprintf(cadena,"%cMode: %s", (opt == 0) ? '-' : ' ',(mode == 0) ? "Solid mask" : ( (mode == 1) ? "Tiles" : "Objects" ));
	draw_text(xorg + camera_x, camera_y + 350, cadena, tex_digits);
	
	//Tileset
	sprintf(cadena,"%cTileset: %d", (opt == 1) ? '-' : ' ', *tileset);
	draw_text(xorg + camera_x, camera_y + 360, cadena, tex_digits);
	
	char object_name[2][20] = { "Enemy dog", "Go to room" };
	
	//Object
	sprintf(cadena,"%cObject: %s", (opt == 2) ? '-' : ' ', object_name[object - 1]);
	draw_text(xorg + camera_x, camera_y + 370, cadena, tex_digits);
	
	//Showing grid?
	sprintf(cadena,"%cGrid: %s", (opt == 3) ? '-' : ' ',(show_grid) ? "ON" : "OFF");
	draw_text(xorg + camera_x, camera_y + 380, cadena, tex_digits);
	
	//Showing tiles?
	sprintf(cadena,"%cTiles: %s", (opt == 4) ? '-' : ' ', (show_tiles) ? "ON" : "OFF");
	draw_text(xorg + camera_x, camera_y + 390, cadena, tex_digits);	
	
	//Showing solid mask?
	sprintf(cadena,"%cSolid: %s", (opt == 5) ? '-' : ' ', (show_solid) ? "ON" : "OFF");
	draw_text(xorg + camera_x, camera_y + 400, cadena, tex_digits);

	//Showing objects
	sprintf(cadena,"%cShow objects %s", (opt == 6) ? '-' : ' ', (show_objects) ? "ON" : "OFF");
	draw_text(xorg + camera_x, camera_y + 410, cadena, tex_digits);

	//Draw level's size
	sprintf(cadena,"%croom_width: %d", (opt == 7) ? '-' : ' ', level_w);
	draw_text(xorg + camera_x, camera_y + 420, cadena, tex_digits);
	
	//Draw level's size
	sprintf(cadena,"%croom_height: %d", (opt == 8) ? '-' : ' ', level_h);
	draw_text(xorg + camera_x, camera_y + 430, cadena, tex_digits);	

	//Draw mouse position
	sprintf(cadena,"%.4d %.4d", *mx, *my);
	draw_text(xorg + camera_x, camera_y + 450, cadena, tex_digits);
	
	//Draw camera position
	sprintf(cadena,"%.4d %.4d", camera_x, camera_y);
	draw_text(xorg + camera_x, camera_y + 460, cadena, tex_digits);	
}

void draw_tiles(unsigned char tile_grid[MAX_LEVEL_W/16][MAX_LEVEL_H/16], GLuint  * tex_tileset, char tileset)
{
	glColor3f(1.0,1.0,1.0);				

	for( char i = 0; i < MAX_LEVEL_W/16; i++ )
	{
		for( char j = 0; j < MAX_LEVEL_H/16; j++ )
		{
			Draw_Texture(tex_tileset[tileset], i * 16, j * 16, 16, 16, 1024, 16, tile_grid[i][j]);		
		}
	}
}

void fill_tiles(unsigned char tile_grid[MAX_LEVEL_W/16][MAX_LEVEL_H/16], unsigned int level_w, unsigned int level_h, unsigned int terrain_grid[][9], unsigned int triangles)
{
	/*
		This function fills empty cells in the tile_grid with an according tile,
		and is meant to reduce time needed for level design.
	*/
	for( char i = 0; i < level_w/16; i++ )
	{
		for( char j = 0; j < level_h/16; j++ )
		{
			if ( tile_grid[i][j] == 0 ) // we only auto-tile on free cells
			{
				bool free[6] = { 0, 0, 0, 0, 0, 0 };
				for( int k = 1; k < triangles; k++ )
				{				
					if ( in_triangle( terrain_grid, k, i * 16, j * 16 + 2 ) )
						free[0] = 1;
					if ( in_triangle( terrain_grid, k, i * 16 + 16, j * 16 ) )
						free[1] = 1;
					if ( in_triangle( terrain_grid, k, i * 16 + 16, j * 16 + 8 ) )
						free[2] = 1;						
					if ( in_triangle( terrain_grid, k, i * 16 + 16, j * 16 + 16 ) )
						free[3] = 1;
					if ( in_triangle( terrain_grid, k, i * 16, j * 16 + 16) )
						free[4] = 1;						
					if ( in_triangle( terrain_grid, k, i * 16, j * 16 + 8) )
						free[5] = 1;												
				}
				if ( !free[0] && !free[1] && !free[2] && !free[3] && !free[4] && !free[5] )
					tile_grid[i][j] = 12 + (int) random( 7.0 );
				else if ( free[0] && free[1] && free[2] && !free[3] && !free[4] && free[5] )
					tile_grid[i][j] = 1 + (int) random( 4.0 );				
				else if ( free[0] && free[1] && !free[2] && !free[3] && !free[4] && !free[5] )
					tile_grid[i][j] = 6 + (int) random( 5.0 );									
				else if ( !free[0] && !free[1] && free[2] && free[3] && free[4] && free[5] )
					tile_grid[i][j] = 24 + (int) random( 4.0 );
				else if ( !free[0] && free[1] && free[2] && !free[3] && !free[4] && !free[5] )
					tile_grid[i][j] = 39;
				else if ( !free[0] && free[1] && !free[2] && !free[3] && !free[4] && !free[5] )
					tile_grid[i][j] = 40 + (int) random( 3.0 );				
				else if ( free[0] && free[1] && free[2] && !free[3] && !free[4] && !free[5] )				
					tile_grid[i][j] = 44 + (int) random( 2.0 );					
				else if ( free[0] && !free[1] && !free[2] && !free[3] && !free[4] && free[5] )
					tile_grid[i][j] = 47;
				else if ( free[0] && !free[1] && !free[2] && !free[3] && !free[4] && !free[5] )
					tile_grid[i][j] = 48 + (int) random( 3.0 );					
				else if ( free[0] && free[1] && !free[2] && !free[3] && !free[4] && free[5] )
					tile_grid[i][j] = 52 + (int) random( 3.0 );						
			}
		}
	}
				
}

void draw_text(unsigned int x, unsigned int y, char cadena[], GLuint tex_digits)
{
	for(char i = 0; i < strlen(cadena); i++)
		Draw_Texture(tex_digits, x + i*6, y, 8, 8, 1024, 8, cadena[i] - 33);
}		
