#include <SDL/SDL_opengl.h>
#include <SDL/SDL.h>
#include "../headers/functions.h"
#include "../headers/gmath.h"
#include <string.h>

#include "../headers/PlayerClass.h"
#include "../headers/EnemyDogClass.h"
#include "../headers/GoToRoomClass.h"

extern EnemyDog myenemydog;
extern Player myplayer;
extern GoToRoom mygotoroom;

void load_terrain(unsigned int terrain_grid[][9], unsigned int *triangles, unsigned int *level_w, unsigned int *level_h, unsigned char tile_grid[MAX_LEVEL_W/16][MAX_LEVEL_H/16], unsigned char * tileset, unsigned int level )
{
	FILE* file;
	char filepath[30] = "resources/";
    char levelname[2][20] = { "level0", "level1" };        
    
	strcat( filepath, levelname[level] );
	file = fopen( filepath,"rb");
	//Room dimensions
    fread( level_w, sizeof(int), 1, file ); 
    fread( level_h, sizeof(int), 1, file ); 	
	//Amount of triangles
    fread( triangles, sizeof(int), 1, file ); 
    //For every triangle
	for(char i = 1; i < *triangles; i++)
	{
		//Read 9 values (3 vertices -> 6 values) + 3 adjents
	    fread( &terrain_grid[i][0], sizeof(int), 9, file );			    
	}
	
	//Read tiles
	fread(tileset, sizeof(char), 1, file);
	for(unsigned char i = 0; i < *level_w/16; i++)
		for(unsigned char j = 0; j < *level_h/16; j++)
			fread(&tile_grid[i][j], sizeof(char), 1, file);		
	
	//Place player
	if ( level == 0 )
	{
		myplayer.set_xy(1004, 180);
		myplayer.set_my_triangles(terrain_grid, *triangles);
	}
	else if ( level == 1 )
	{
		myplayer.set_xy(20, 180);
		myplayer.set_my_triangles(terrain_grid, *triangles);
	}
			
	int temp;			
	fread( &temp, sizeof(unsigned int), 1, file);
	
	
	while ( !feof(file) )
	{
/*	
FILE *f;

f = fopen("jojojo.txt","a");
fprintf(f,"%d",temp);
	
fclose(f);	
*/		
		int x, y;
		switch( temp )
		{
			case 1: //enemydog
				fread( &x, sizeof(unsigned int), 1, file);
				fread( &y, sizeof(unsigned int), 1, file);				
				myenemydog.create( x, y );
				myenemydog.set_my_triangles(terrain_grid, *triangles);
			break;
			case 2: //gotoroom
				fread( &x, sizeof(unsigned int), 1, file);
				fread( &y, sizeof(unsigned int), 1, file);
				mygotoroom.create( x, y );
			break;		
		}	
		fread( &temp, sizeof(unsigned int), 1, file);	
	}				
	
	fclose(file);
}

bool place_free(unsigned int terrain_grid[][9], unsigned int xprev, unsigned int yprev, unsigned int x, unsigned int y, int triangle)
{
	if (x < 4)
		return 0;
	
	if (in_triangle(terrain_grid, triangle, x, y)) //if still on same triangle					
		return 1;
	
	//Calculate which side on our current triangle our movement vector is colliding with
	double dir[3];
	for(char i = 0; i < 3; i++)	//check against 3 vertices in current triangle
	{
		if ( point_distance( xprev, yprev, terrain_grid[triangle][i * 2], terrain_grid[triangle][i * 2 + 1]) != 0 )
			dir[i] = point_direction2( xprev, yprev, terrain_grid[triangle][i * 2], terrain_grid[triangle][i * 2 + 1]);
		else
		{
			double aux_dir[2];
			switch ( i )
			{
				case 0:				
					aux_dir[0] = point_direction2( xprev, yprev, terrain_grid[triangle][2], terrain_grid[triangle][3] );
					aux_dir[1] = point_direction2( xprev, yprev, terrain_grid[triangle][4], terrain_grid[triangle][5] );
				break;
				case 1:
					aux_dir[0] = point_direction2( xprev, yprev, terrain_grid[triangle][0], terrain_grid[triangle][1] );
					aux_dir[1] = point_direction2( xprev, yprev, terrain_grid[triangle][4], terrain_grid[triangle][5] );
				break;
				case 2:
					aux_dir[0] = point_direction2( xprev, yprev, terrain_grid[triangle][0], terrain_grid[triangle][1] );
					aux_dir[1] = point_direction2( xprev, yprev, terrain_grid[triangle][2], terrain_grid[triangle][3] );					
				break;					
			}
			
			dir[i] = (aux_dir[0] + aux_dir[1]) / 2;
			if ( angle_diff( aux_dir[0], dir[i] ) < degtorad(90.0) )
				dir[i] += degtorad(180);
			while( dir[i] > degtorad(360) )
				dir[i] -= degtorad(360);
		}
	}			
	
	double our_dir = point_direction2( xprev, yprev, x, y );	
	
	char ordered_dirs[3];
	ordered_dirs[0] = ( dir[0] <= dir[1] && dir[0] <= dir[2] ) ? 0 : ( ( dir[1] <= dir[2] ) ? 1 : 2 );
	ordered_dirs[2] = ( dir[0] >= dir[1] && dir[0] >= dir[2] ) ? 0 : ( ( dir[1] >= dir[2] ) ? 1 : 2 );
	ordered_dirs[1] = 0;
	
	if ( ordered_dirs[0] == 0 || ordered_dirs[2] == 0 )
	{
		ordered_dirs[1] = 1;
		if ( ordered_dirs[0] == 1 || ordered_dirs[2] == 1 )
			ordered_dirs[1] = 2;	
	}
	
	int a1, b1, a2, b2;
	
	if ( our_dir <= dir[ordered_dirs[1]] && our_dir >= dir[ordered_dirs[0]] )
	{
		a1 = terrain_grid[triangle][ordered_dirs[0] * 2];
		b1 = terrain_grid[triangle][ordered_dirs[0] * 2 + 1];
		a2 = terrain_grid[triangle][ordered_dirs[1] * 2];
		b2 = terrain_grid[triangle][ordered_dirs[1] * 2 + 1];		
	}
	else if ( our_dir <= dir[ordered_dirs[2]] && our_dir >= dir[ordered_dirs[1]] )
	{
		a1 = terrain_grid[triangle][ordered_dirs[1] * 2];
		b1 = terrain_grid[triangle][ordered_dirs[1] * 2 + 1];
		a2 = terrain_grid[triangle][ordered_dirs[2] * 2];
		b2 = terrain_grid[triangle][ordered_dirs[2] * 2 + 1];		
	}
	else
	{
		a1 = terrain_grid[triangle][ordered_dirs[0] * 2];
		b1 = terrain_grid[triangle][ordered_dirs[0] * 2 + 1];
		a2 = terrain_grid[triangle][ordered_dirs[2] * 2];
		b2 = terrain_grid[triangle][ordered_dirs[2] * 2 + 1];		
	}	
	
	for(char i = 0; i < 3; i++)	//check against 3 sides in current triangle
	{
		//get vertices of the side
		int a1 = terrain_grid[triangle][i * 2];
		int b1 = terrain_grid[triangle][i * 2 + 1];
		int a2, b2;
		if (i < 2)
		{
			a2 = terrain_grid[triangle][(i + 1) * 2];
			b2 = terrain_grid[triangle][(i + 1) * 2 + 1];		
		}
		else
		{
			a2 = terrain_grid[triangle][0];
			b2 = terrain_grid[triangle][1];		
		}
	}
				
	double xcol, ycol;
	if (segment_collision(xprev, yprev, x, y, a1, b1, a2, b2, &xcol, &ycol))
	{
		//if the side we are colliding with is a side of an adjent triangle
		for ( char k = 6; k <= 8; k++ ) //for every adjent		
		{
			if ( terrain_grid[triangle][k] != 0 )
			{
				if ( is_side_of_triangle(terrain_grid, terrain_grid[triangle][k], a1, b1, a2, b2) )
				{										
					return place_free(terrain_grid, (unsigned int)xcol, (unsigned int)ycol, x, y, terrain_grid[triangle][k]);
				}
			}
		}
	}
	return 0;
}

bool place_free(unsigned int terrain_grid[][9], unsigned int xprev, unsigned int yprev, unsigned int x, unsigned int y, int next_triangle[4], char vertex )
{	
	int triangle = next_triangle[vertex];	
	
	if (x < 4)
		return 0;
	
	if (in_triangle(terrain_grid, triangle, x, y)) //if still on same triangle					
		return 1;	
	
	//Calculate which side on our current triangle our movement vector is colliding with
	double dir[3];
	for(char i = 0; i < 3; i++)	//check against 3 vertices in current triangle
	{
		if ( point_distance( xprev, yprev, terrain_grid[triangle][i * 2], terrain_grid[triangle][i * 2 + 1]) != 0 )
			dir[i] = point_direction2( xprev, yprev, terrain_grid[triangle][i * 2], terrain_grid[triangle][i * 2 + 1]);
		else
		{
			double aux_dir[2];
			switch ( i )
			{
				case 0:				
					aux_dir[0] = point_direction2( xprev, yprev, terrain_grid[triangle][2], terrain_grid[triangle][3] );
					aux_dir[1] = point_direction2( xprev, yprev, terrain_grid[triangle][4], terrain_grid[triangle][5] );
				break;
				case 1:
					aux_dir[0] = point_direction2( xprev, yprev, terrain_grid[triangle][0], terrain_grid[triangle][1] );
					aux_dir[1] = point_direction2( xprev, yprev, terrain_grid[triangle][4], terrain_grid[triangle][5] );
				break;
				case 2:
					aux_dir[0] = point_direction2( xprev, yprev, terrain_grid[triangle][0], terrain_grid[triangle][1] );
					aux_dir[1] = point_direction2( xprev, yprev, terrain_grid[triangle][2], terrain_grid[triangle][3] );					
				break;					
			}
			
			dir[i] = (aux_dir[0] + aux_dir[1]) / 2;
			if ( angle_diff( aux_dir[0], dir[i] ) < degtorad(90.0) )
				dir[i] += degtorad(180);
			while( dir[i] > degtorad(360) )
				dir[i] -= degtorad(360);
		}
	}			
	
	double our_dir = point_direction2( xprev, yprev, x, y );	
	
	char ordered_dirs[3];
	ordered_dirs[0] = ( dir[0] <= dir[1] && dir[0] <= dir[2] ) ? 0 : ( ( dir[1] <= dir[2] ) ? 1 : 2 );
	ordered_dirs[2] = ( dir[0] >= dir[1] && dir[0] >= dir[2] ) ? 0 : ( ( dir[1] >= dir[2] ) ? 1 : 2 );
	ordered_dirs[1] = 0;
	
	if ( ordered_dirs[0] == 0 || ordered_dirs[2] == 0 )
	{
		ordered_dirs[1] = 1;
		if ( ordered_dirs[0] == 1 || ordered_dirs[2] == 1 )
			ordered_dirs[1] = 2;	
	}		
	
	int a1, b1, a2, b2;
	
	if ( our_dir <= dir[ordered_dirs[1]] && our_dir >= dir[ordered_dirs[0]] )
	{
		a1 = terrain_grid[triangle][ordered_dirs[0] * 2];
		b1 = terrain_grid[triangle][ordered_dirs[0] * 2 + 1];
		a2 = terrain_grid[triangle][ordered_dirs[1] * 2];
		b2 = terrain_grid[triangle][ordered_dirs[1] * 2 + 1];		
	}
	else if ( our_dir <= dir[ordered_dirs[2]] && our_dir >= dir[ordered_dirs[1]] )
	{
		a1 = terrain_grid[triangle][ordered_dirs[1] * 2];
		b1 = terrain_grid[triangle][ordered_dirs[1] * 2 + 1];
		a2 = terrain_grid[triangle][ordered_dirs[2] * 2];
		b2 = terrain_grid[triangle][ordered_dirs[2] * 2 + 1];		
	}
	else
	{
		a1 = terrain_grid[triangle][ordered_dirs[0] * 2];
		b1 = terrain_grid[triangle][ordered_dirs[0] * 2 + 1];
		a2 = terrain_grid[triangle][ordered_dirs[2] * 2];
		b2 = terrain_grid[triangle][ordered_dirs[2] * 2 + 1];		
	}			
	
	double xcol, ycol;		

	
	if (segment_collision(xprev, yprev, x, y, a1, b1, a2, b2, &xcol, &ycol))
	{									
		//if the side we are colliding with is a side of an adjent triangle
		for ( char k = 6; k <= 8; k++ ) //for every adjent		
		{
			if ( terrain_grid[triangle][k] != 0 )
			{
				if ( is_side_of_triangle(terrain_grid, terrain_grid[triangle][k], a1, b1, a2, b2) )
				{		
																														
					unsigned int myxcol = (unsigned int) xcol;
					unsigned int myycol = (unsigned int) ycol;
					if (!in_triangle( terrain_grid, terrain_grid[triangle][k], myxcol, myycol ) )		
					{
						for (char i = -1; i <= 1; i += 2 )
						{
							for (char j = -1; j <= 1; j += 2 )						
							{
								if (in_triangle( terrain_grid, terrain_grid[triangle][k], myxcol + i, myycol + j ) )										
								{
									myxcol += i;
									myycol += j;
									break;
								}
							}
						}
					}
					
					next_triangle[vertex] = terrain_grid[triangle][k];					
																															
					return (place_free(terrain_grid, myxcol, myycol, x, y, next_triangle, vertex));					
				}
			}
		}
	}
	
	return 0;
}

bool line_free(unsigned int terrain_grid[1][9], unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, unsigned int triangle)
{						
	
	if (in_triangle(terrain_grid, triangle, x2, y2))
		return 1;
	
	double dir[3];
	for(char i = 0; i < 3; i++)	//check against 3 vertices in current triangle
	{
		if ( point_distance( x1, y1, terrain_grid[triangle][i * 2], terrain_grid[triangle][i * 2 + 1]) != 0 )
			dir[i] = point_direction2( x1, y1, terrain_grid[triangle][i * 2], terrain_grid[triangle][i * 2 + 1]);
		else
		{
			double aux_dir[2];
			switch ( i )
			{
				case 0:				
					aux_dir[0] = point_direction2( x1, y1, terrain_grid[triangle][2], terrain_grid[triangle][3] );
					aux_dir[1] = point_direction2( x1, y1, terrain_grid[triangle][4], terrain_grid[triangle][5] );
				break;
				case 1:
					aux_dir[0] = point_direction2( x1, y1, terrain_grid[triangle][0], terrain_grid[triangle][1] );
					aux_dir[1] = point_direction2( x1, y1, terrain_grid[triangle][4], terrain_grid[triangle][5] );
				break;
				case 2:
					aux_dir[0] = point_direction2( x1, y1, terrain_grid[triangle][0], terrain_grid[triangle][1] );
					aux_dir[1] = point_direction2( x1, y1, terrain_grid[triangle][2], terrain_grid[triangle][3] );					
				break;					
			}
			
			dir[i] = (aux_dir[0] + aux_dir[1]) / 2;
			if ( angle_diff( aux_dir[0], dir[i] ) < degtorad(90.0) )
				dir[i] += degtorad(180);
			while( dir[i] > degtorad(360) )
				dir[i] -= degtorad(360);
		}
	}			
	
	double our_dir = point_direction2( x1, y1, x2, y2 );	
	
	char ordered_dirs[3];
	ordered_dirs[0] = ( dir[0] <= dir[1] && dir[0] <= dir[2] ) ? 0 : ( ( dir[1] <= dir[2] ) ? 1 : 2 );
	ordered_dirs[2] = ( dir[0] >= dir[1] && dir[0] >= dir[2] ) ? 0 : ( ( dir[1] >= dir[2] ) ? 1 : 2 );
	ordered_dirs[1] = 0;
	
	if ( ordered_dirs[0] == 0 || ordered_dirs[2] == 0 )
	{
		ordered_dirs[1] = 1;
		if ( ordered_dirs[0] == 1 || ordered_dirs[2] == 1 )
			ordered_dirs[1] = 2;	
	}		
	
	int a1, b1, a2, b2;
	
	if ( our_dir <= dir[ordered_dirs[1]] && our_dir >= dir[ordered_dirs[0]] )
	{
		a1 = terrain_grid[triangle][ordered_dirs[0] * 2];
		b1 = terrain_grid[triangle][ordered_dirs[0] * 2 + 1];
		a2 = terrain_grid[triangle][ordered_dirs[1] * 2];
		b2 = terrain_grid[triangle][ordered_dirs[1] * 2 + 1];		
	}
	else if ( our_dir <= dir[ordered_dirs[2]] && our_dir >= dir[ordered_dirs[1]] )
	{
		a1 = terrain_grid[triangle][ordered_dirs[1] * 2];
		b1 = terrain_grid[triangle][ordered_dirs[1] * 2 + 1];
		a2 = terrain_grid[triangle][ordered_dirs[2] * 2];
		b2 = terrain_grid[triangle][ordered_dirs[2] * 2 + 1];		
	}
	else
	{
		a1 = terrain_grid[triangle][ordered_dirs[0] * 2];
		b1 = terrain_grid[triangle][ordered_dirs[0] * 2 + 1];
		a2 = terrain_grid[triangle][ordered_dirs[2] * 2];
		b2 = terrain_grid[triangle][ordered_dirs[2] * 2 + 1];		
	}			
	
	double x, y;		
	
	if (segment_collision(x1, y1, x2, y2, a1, b1, a2, b2, &x, &y))		
	{	

														
/*glBegin( GL_LINES );
		
glColor3f(1.0,1.0,1.0);
		
glVertex2f( x1, y1 );
glVertex2f( x, y );		

glColor3f(1.0,1.0,1.0);
    		
glEnd();*/

							
							
/*Uint8 *keystates = SDL_GetKeyState( NULL );	
//if ( keystates[ SDLK_b ] && x1 == 265)
{
	FILE *f;

	f = fopen("jojojo.txt","a");
	fprintf(f,"\nsee_player(%d,%d,%d,%d,%d)",x1,y1,x2,y2,triangle);
	fprintf(f,"\ncollision with: (%d, %d)-(%d, %d)", a1, b1, a2, b2);
	fprintf(f,"\ncollision at: (%f, %f)"/* trims to (%d, %d)\n, x, y/* myxcol, myycol);
	fclose(f);
}										*/
								
								
		//there is a collision
		//if the collision takes place with a side that is shared with another triangle, we move on to that triangle
		for(char j = 0; j < 3; j++) //for every adjent
		{
			if(terrain_grid[triangle][6 + j] != 0)
			{
				if ( is_side_of_triangle(terrain_grid, terrain_grid[triangle][6 + j], a1, b1, a2, b2) )								
				{						
					unsigned int myxcol = (unsigned int) x;
					unsigned int myycol = (unsigned int) y;
					if (!in_triangle( terrain_grid, terrain_grid[triangle][6 + j], myxcol, myycol ) )		
					{
						for (char a = -1; a <= 1; a += 2 )
						{
							for (char b = -1; b <= 1; b += 2 )						
							{
								if (in_triangle( terrain_grid, terrain_grid[triangle][6 + j], myxcol + a, myycol + b ) )										
								{
									myxcol += a;
									myycol += b;
									break;
								}
							}
						}
					}																						
					return line_free(terrain_grid, myxcol, myycol, x2, y2, terrain_grid[triangle][6 + j]);						
				}
			}
		}
	}

	return (in_triangle(terrain_grid, triangle, x2, y2));
}

bool in_triangle(unsigned int terrain_grid[][9], unsigned int triangle, unsigned int x , unsigned int y)
{	
	return (abs(     
		abs((int)((terrain_grid[triangle][0] - x) * (terrain_grid[triangle][3] - y) - (terrain_grid[triangle][2] - x) * (terrain_grid[triangle][1] - y)))
		+ 
		abs((int)((terrain_grid[triangle][2] - x) * (terrain_grid[triangle][5] - y) - (terrain_grid[triangle][4] - x) * (terrain_grid[triangle][3] - y))) 
		+ 
		abs((int)((terrain_grid[triangle][4] - x) * (terrain_grid[triangle][1] - y) - (terrain_grid[triangle][0] - x) * (terrain_grid[triangle][5] - y))) 
		-	
		abs((int)((terrain_grid[triangle][2] - terrain_grid[triangle][0]) * (terrain_grid[triangle][5] - terrain_grid[triangle][1]) - (terrain_grid[triangle][4] - terrain_grid[triangle][0]) * (terrain_grid[triangle][3] - terrain_grid[triangle][1])))
		) <= (1 / 256));
}

bool draw_terrain(unsigned int terrain_grid[][9], unsigned int triangles)
{
    	glDisable(GL_TEXTURE_2D);
		glBegin( GL_TRIANGLES );
		
		glColor3f(1.0,1.0,1.0);
		
		// Draw terrain triangles
		for(char i = 1; i < triangles; i++)
		{
			glVertex3f( terrain_grid[i][0], terrain_grid[i][1], 0 );
			glVertex3f( terrain_grid[i][2], terrain_grid[i][3], 0 );
	    	glVertex3f( terrain_grid[i][4], terrain_grid[i][5], 0 );		
		}
		    
		glEnd();	
}


void set_triangles(unsigned int terrain_grid[1][9], int* current, int x, int y, char w, char h, unsigned int triangles)
{
	for (char i = 0; i < 4; i++)
	{
		for (int j = 0; j < triangles; j++ )
		{
			if ( in_triangle(terrain_grid, j, x - w / 2 + (i == 1 || i == 2) * w, y - h / 2 + (i == 2 || i == 3) * h ) )
				current[i] = j;
		}
	}
}

void set_triangle(unsigned int terrain_grid[1][9], int* current, int x, int y, unsigned int triangles)
{
	for (int j = 0; j < triangles; j++ )
	{
		if ( in_triangle(terrain_grid, j, x, y ) )
			*current = j;
	}
}

bool segment_collision( unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, unsigned int a1, unsigned int b1, unsigned int a2, unsigned int b2, double * xcol, double * ycol)
{
		
	if ( x1 != x2 && a1 != a2 ) //none of them are vertical
	{		
		//get m of 1st line
		double m1 = ((double)y2 - (double)y1) / (double)((double)x2 - (double)x1);
		//get m of the 2nd line
		double m2 = double((double)b2 - (double)b1) / double((double)a2 - (double)a1);
		//get x coodinate where collision occurs
		double x = (m1 * (double)x1 - m2 * (double)a1 - (double)y1 + (double)b1) / (m1 - m2);
		//get y coordinate where the collision occurs	
		double y = y1 + m1 * (x - x1);
		
		*xcol = x;
		*ycol = y;
	
		//check if the collision takes place among the limits of the side AND of the segment		
		return ( ( ( ((unsigned int)x >= a1 && (unsigned int)x <= a2 ) || ((unsigned int)x <= a1 && (unsigned int)x >= a2) ) && ( ((unsigned int)y >= b2 && (unsigned int)y <= b1) || ((unsigned int)y >= b1 && (unsigned int)y <= b2) ) )
		&& ( ( ((unsigned int)x >= x1 && (unsigned int)x <= x2 ) || ((unsigned int)x <= x1 && (unsigned int)x >= x2) ) && ( ((unsigned int)y >= y2 && (unsigned int)y <= y1) || ((unsigned int)y >= y1 && (unsigned int)y <= y2) ) ) );
	}
	else if ( x1 == x2 && a1 != a2 )
	{
		if ( ( x1 >= a1 && x1 <= a2 ) || ( x1 <= a1 && x1 >= a2 ) )
		{
			//get m of the 2nd line
			double m2 = double((double)b2 - (double)b1) / double((double)a2 - (double)a1);		
			double y = (double)b1 + m2 * ( (double)x1 - (double)a1 );						

			*xcol = x1;
			*ycol = y;
			
			return ( ( (unsigned int)y >= y1 && (unsigned int)y <= y2 ) || ( (unsigned int)y <= y1 && (unsigned int)y >= y2 ) );
		}
		else 
			return 0;		
	}
	else if ( x1 != x2 && a1 == a2 )
	{
		if ( ( a1 >= x1 && a1 <= x2 ) || ( a1 <= x1 && a1 >= x2 ) )
		{
			//get m of the 1st line
			double m1 = double((double)y2 - (double)y1) / double((double)x2 - (double)x1);		
			double y = (double)y1 + m1 * ( (double)a1 - (double)x1 );
			*xcol = x1;
			*ycol = y;			
			return ( ( (unsigned int)y >= b1 && (unsigned int)y <= b2 ) || ( (unsigned int)y <= b1 && (unsigned int)y >= b2 ) );
		}
		else 
			return 0;		
	}	
}

bool is_side_of_triangle(unsigned int terrain_grid[][9], unsigned int triangle, unsigned int a1, unsigned int b1, unsigned int a2, unsigned int b2)
{
	unsigned int x1 = terrain_grid[triangle][0];
	unsigned int y1 = terrain_grid[triangle][1];
	unsigned int x2 = terrain_grid[triangle][2];
	unsigned int y2 = terrain_grid[triangle][3];
	unsigned int x3 = terrain_grid[triangle][4];
	unsigned int y3 = terrain_grid[triangle][5];				
	
	if ( ( a1 == x1 && b1 == y1 ) || ( a1 == x2 && b1 == y2 ) || ( a1 == x3 && b1 == y3 ) )
		if ( ( a2 == x1 && b2 == y1 ) || ( a2 == x2 && b2 == y2 ) || ( a2 == x3 && b2 == y3 ) )	
			return 1;
			
	return 0;
}
